# Version 1.2

Version 1.2 upgrades React to v19. A few of the other dependencies were also updated, which will need to be updated in each project.

Please take the following steps when upgrading:

- Update `react`, `react-dom` and their types packages to the latest version.
- Update `react-select` to at least 5.10.0.
- Update `react-custom-scrollbars` and its types to the latest version.
- If you are using `rc-slider`. 
    - Remove the `@types/rc-slider` package, and update `rc-slider` to the latest version.
    - In your app module, remove the `Range` import, and remove this line: `container.bind(Types.Neo.Components.Range).toConstantValue(Range);`
- If you are using `react-color`.
    - Update the package, and types to the latest version.
- Under React 19, JSX is no longer a global import. You may receive errors such as "Cannot find namespace 'JSX'." 
    - To rectify, change `JSX.Element` to `React.ReactNode`.
    - If that doesn't work, you can import JSX from React. E.g. `import React, { JSX } from 'React';`
- Finally, run `yarn dedupe` to make sure all old transitive packages reference the project version.

# Version 1.1

Version 1.1 removes the `react-router` dependency. This requires updates to your project which are explained in [this guide](./docs/RemoveReactRouter.md).

It also fixes some issues preventing projects from upgrading to `mobx-react` v9.x.

## Version 1.0.22

React-select dependency was updated which has a breaking change in its types. You will need to update the version of this package in your project to at least:

```json
"react-select": "5.8.1",
```

# Version 1.0

Version 1 of neo-react upgrades react to version 18, and mobX to version 6. Both of these are major updates and will require you to make code changes to your project.

## Testing

In addition to requiring code / syntax changes, both React 18, and Mobx 6 behave differently enough to their previous versions that runtime bugs may be introduced into your project. You should only update to this version when you and the client have enough time to test the project thoroughly. If you find any issues, please report them so this document can be updated.

The project changes required are detailed below:

## Package changes:

* Update all neo packages to the latest 1.x version.
* Update the following packages to the latest version (these were the latest at time of writing):
    * dependencies
        * `"axios": "1.6.7"`,
        * `"decimal.js-light": "2.5.1"`, - See below for alternative
        * `"mobx": "6.11.0"`,
        * `"mobx-react": "7.6.0"`,
        * `"react": "18.2.0"`,
        * `"react-dom": "18.2.0"`,
        * `"react-router": "5.2.1"`, - Must be version 5.x
        * `"react-router-dom": "5.3.0"`, - Must be version 5.x
        * `"react-select": "5.7.3"`,
        
    * devDependencies
        * `"@types/react": "18.2.37"`,
        * `"@types/react-dom": "18.2.15"`,
        * `"@types/react-router": "5.1.17"`, - Must be version 5.x
        * `"@types/react-router-dom": "5.3.3"`, - Must be version 5.x
    * resolutions
        * remove all entries
* delete yarn.lock, and run `yarn install`

## Fix errors

If any of your components allow children, React 18 requires to you have an explicit `children` prop. Add this to the props definition of the component:

```typescript
interface MyProps {
    ...
    children: React.ReactNode;
}

// or if the component has no other props:

MyComponent extends React.Component<{ children: React.ReactNode }> {
    ...
}
```

The React root api has also changed in version 18. Make the following changes to your `index.tsx` file:

Your existing code will look something like this:

```typescript
import ReactDOM from 'react-dom';
...
// React init
ReactDOM.render(
    renderComponent,
    document.getElementById('root') as HTMLElement
);
```

Change the `ReactDOM` import, and the react init code:

```typescript
import ReactDOM from 'react-dom/client';
...
// React init
const root = ReactDOM.createRoot(document.getElementById('root') as HTMLElement);
root.render(renderComponent);
```

## Mobx

Before mobx 6, you could mark a property as being observable by adding an `@observable` decorator to the property. In mobx 6 this no longer works ([read more](https://mobx.js.org/migrating-from-4-or-5.html)). 

For any class that extends `ModelBase`, `ValueObject`, `ViewModelBase` etc, this change is handled by neo, and you will not need to change any code.

For other classes, you will need to add initialisation code to the constructor.

* Search for `@observable` in your project. (In most projects this will be in the Sidebar and AppLayout components)
* In the constructor of the class that contains the property, add:

```typescript
makeObservable(this);
```

### Decorators on neo models

If you try use mobx decorators on any models that extend `ModelBase`, `ValueObject` or `ViewModelBase`, you will receive this error:

> [MobX] makeObservable second arg must be nullish when using decorators. Mixing @decorator syntax with annotations is not supported.

This is most likely because you have used an `@action` decorator on a method. Neo now marks all methods as actions, so you can remove the `@action` decorator.

* Search for `@action` in your project. Remove the decorator **only** if the class extends `ModelBase` (and its decendents e.g. `ValueObject`) or `ViewModelBase`.

### Array sorting

Mobx no longer allows you to call `sort` on an observable array. You first need to call `slice` to create a copy of the array. This change has been made in the neo `sortBy` method, but will cause unexpected behaviour if you relied on the list being sorted in-place, as this no longer happens.

* Search for `.sort` in your project. Change to use the neo `sortBy` method.
* Search for `.sortBy`. Ensure you use the array returned by this method, and not the array passed in.

### Viewmodels

You may receive errors such as:

```
Argument of type 'typeof MyVM' is not assignable to parameter of type 'VMConstructor<MyVM>'.
```

The `registerViewModel` method on the `ViewModelBase` class now requires the view model passed to it to have a `taskRunner` as the first argument. This task runner instance must be passed to the base constructor.

## Decimals

Neo no longer includes the `decimal.js-light` package by default. This is to allow for other decimal packages to be used. You will now need to manually register a decimal service in your app module.

The easiest would be to continue using `decimal.js-light`:

* Ensure your project references `decimal.js-light` directly.
* In your main app module, add the following code:
```typescript
import Decimal from "decimal.js-light";
...
container.bind(Types.Neo.Core.DecimalService).toConstantValue(new Numeric.DecimalService(Decimal, Decimal.ROUND_HALF_UP, Decimal.ROUND_DOWN));
```

An alternative is to use `big.js`. This package is slightly smaller than the above, but has less features.

```typescript
import Big from "big.js";
...
container.bind(Types.Neo.Core.DecimalService).toConstantValue(new Numeric.DecimalService(Big, Big.roundHalfUp, Big.roundDown))
```